/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "iqrf.h"

// ------------------------------------------------------------- PRIVATE MACROS 


// -------------------------------------------------------------- PRIVATE TYPES


// ------------------------------------------------------------------ CONSTANTS


// ------------------------------------------------------------------ VARIABLES


// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

static void dev_comm_delay ( void );

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void iqrf_cfg_setup ( iqrf_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->rx_pin     = HAL_PIN_NC;
    cfg->tx_pin     = HAL_PIN_NC;
    
    // Additional gpio pins

    cfg->adc        = HAL_PIN_NC;
    cfg->io         = HAL_PIN_NC;
    cfg->cs         = HAL_PIN_NC;
    cfg->pwm        = HAL_PIN_NC;
    cfg->int_pin    = HAL_PIN_NC;

    cfg->baud_rate  = 9600;
    cfg->data_bit   = UART_DEFAULT_DATA_BITS;
    cfg->parity_bit = UART_DEFAULT_PARITY_BIT;
    cfg->stop_bit   = UART_DEFAULT_STOP_BIT;
}

IQRF_RETVAL iqrf_init ( iqrf_t *ctx, iqrf_cfg_t *cfg )
{
    uart_config_t uart_cfg;

    uart_configure_default( &uart_cfg );
    uart_cfg.pins.tx_pin = cfg->tx_pin;
    uart_cfg.pins.rx_pin = cfg->rx_pin;
    uart_cfg.baud        = cfg->baud_rate;
    uart_cfg.data_bits   = cfg->data_bit;
    uart_cfg.parity      = cfg->parity_bit;
    uart_cfg.stop        = cfg->stop_bit;

    if ( uart_open( &ctx->uart, &uart_cfg ) != UART_SUCCESS )
    {
        return IQRF_INIT_ERROR;
    }

    // Output pins 

    digital_out_init( &ctx->io, cfg->io );
    digital_out_init( &ctx->cs, cfg->cs );
    digital_out_init( &ctx->pwm, cfg->pwm );

    // Input pins

    digital_in_init( &ctx->adc, cfg->adc );
    digital_in_init( &ctx->int_pin, cfg->int_pin );

    return IQRF_OK;
}

void iqrf_default_cfg ( iqrf_t *ctx )
{
    // Hardware reset
    
    iqrf_hw_reset( ctx );
}

void iqrf_generic_multi_write ( iqrf_t *ctx, iqrf_data_t *data_buf,  uart_length_t len )
{
    uart_block_t block;

    block.buffer = data_buf;
    block.length = len;
    uart_write_block( &ctx->uart, &block);
}

void iqrf_generic_multi_read ( iqrf_t *ctx, iqrf_data_t *data_buf,  uart_length_t len )
{
    uart_block_t block;

    block.buffer = data_buf;
    block.length = len;

    uart_read_block( &ctx->uart, &block);
}

iqrf_data_t iqrf_generic_single_read ( iqrf_t *ctx )
{
    iqrf_data_t rx_data;

    rx_data =  uart_read( &ctx->uart );
    
    return rx_data;
}

void iqrf_generic_single_write ( iqrf_t *ctx, iqrf_data_t tx_data )
{
    uart_write( &ctx->uart, tx_data );
}

uint8_t iqrf_get_interrupt ( iqrf_t *ctx )
{
    return digital_in_read( &ctx->int_pin );
}

void iqrf_hw_reset ( iqrf_t *ctx )
{
    digital_out_high( &ctx->io );
    dev_comm_delay( );
    
    digital_out_low( &ctx->io );
    dev_comm_delay( );
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

static void dev_comm_delay ( void )
{
    Delay_100ms();
}

// ------------------------------------------------------------------------- END

